import os
import sys
import traceback

from manim import logger, console, config, __version__
from manim.utils.module_ops import (
    get_module,
    get_scene_classes_from_module,
    get_scenes_to_render,
    scene_classes_from_file,
)

from manim._config.main_utils import parse_args


def main():
    console.print(f"Manim Community [green]v{__version__}[/green]")
    args = parse_args(sys.argv)

    if hasattr(args, "cmd"):
        if args.cmd == "cfg":
            if args.subcmd:
                from manim._config import cfg_subcmds

                if args.subcmd == "write":
                    cfg_subcmds.write(args.level, args.open)
                elif args.subcmd == "show":
                    cfg_subcmds.show()
                elif args.subcmd == "export":
                    cfg_subcmds.export(args.dir)
            else:
                logger.error("No subcommand provided; Exiting...")

        elif args.cmd == "plugins":
            from manim.plugins import plugins_flags

            if args.list:
                plugins_flags.list_plugins()
            elif not args.list:
                logger.error("No flag provided; Exiting...")

        # elif args.cmd == "some_other_cmd":
        #     something_else_here()

    else:
        config.digest_args(args)
        input_file = config.get_dir("input_file")

        if config["use_opengl_renderer"]:
            from manim.renderer.opengl_renderer import OpenGLRenderer

            for SceneClass in scene_classes_from_file(input_file):
                try:
                    renderer = OpenGLRenderer()
                    scene = SceneClass(renderer)
                    scene.render()
                except Exception:
                    console.print_exception()
        elif config["use_webgl_renderer"]:
            try:
                from manim.grpc.impl import frame_server_impl

                server = frame_server_impl.get(input_file)
                server.start()
                server.wait_for_termination()
            except ModuleNotFoundError:
                console.print(
                    "Dependencies for the WebGL render are missing. Run "
                    "pip install manim[webgl_renderer] to install them."
                )
                console.print_exception()
        else:
            for SceneClass in scene_classes_from_file(input_file):
                try:
                    scene = SceneClass()
                    scene.render()
                except Exception:
                    console.print_exception()


if __name__ == "__main__":
    main()



import math

from ..constants import *
from ..mobject.types.opengl_surface import OpenGLSurface
from ..mobject.types.opengl_surface import OpenGLSurfaceGroup
from ..mobject.types.opengl_vectorized_mobject import OpenGLVGroup
from ..mobject.types.opengl_vectorized_mobject import OpenGLVMobject
from ..utils.space_ops import get_norm
from ..utils.space_ops import z_to_vector


class OpenGLSurfaceMesh(OpenGLVGroup):
    def __init__(
        self,
        uv_surface,
        resolution=None,
        stroke_width=1,
        normal_nudge=1e-2,
        depth_test=True,
        flat_stroke=False,
        **kwargs
    ):
        if not isinstance(uv_surface, OpenGLSurface):
            raise Exception("uv_surface must be of type OpenGLSurface")
        self.uv_surface = uv_surface
        self.resolution = resolution if resolution is not None else (21, 21)
        self.normal_nudge = normal_nudge
        super().__init__(
            stroke_width=stroke_width,
            depth_test=depth_test,
            flat_stroke=flat_stroke,
            **kwargs
        )

    def init_points(self):
        uv_surface = self.uv_surface

        full_nu, full_nv = uv_surface.resolution
        part_nu, part_nv = self.resolution
        u_indices = np.linspace(0, full_nu, part_nu).astype(int)
        v_indices = np.linspace(0, full_nv, part_nv).astype(int)

        points, du_points, dv_points = uv_surface.get_surface_points_and_nudged_points()
        normals = uv_surface.get_unit_normals()
        nudged_points = points + self.normal_nudge * normals

        for ui in u_indices:
            path = OpenGLVMobject()
            full_ui = full_nv * ui
            path.set_points_smoothly(nudged_points[full_ui : full_ui + full_nv])
            self.add(path)
        for vi in v_indices:
            path = OpenGLVMobject()
            path.set_points_smoothly(nudged_points[vi::full_nv])
            self.add(path)


class OpenGLSphere(OpenGLSurface):
    def __init__(self, resolution=None, radius=1, u_range=None, v_range=None, **kwargs):
        resolution = resolution if resolution is not None else (101, 51)
        u_range = u_range if u_range is not None else (0, TAU)
        v_range = v_range if v_range is not None else (0, PI)
        self.radius = radius
        super().__init__(
            resolution=resolution, u_range=u_range, v_range=v_range, **kwargs
        )

    def uv_func(self, u, v):
        return self.radius * np.array(
            [np.cos(u) * np.sin(v), np.sin(u) * np.sin(v), -np.cos(v)]
        )


class OpenGLTorus(OpenGLSurface):
    def __init__(self, u_range=None, v_range=None, r1=3, r2=1, **kwargs):
        u_range = u_range if u_range is not None else (0, TAU)
        v_range = v_range if v_range is not None else (0, TAU)
        self.r1 = r1
        self.r2 = r2
        super().__init__(u_range=u_range, v_range=v_range, **kwargs)

    def uv_func(self, u, v):
        P = np.array([math.cos(u), math.sin(u), 0])
        return (self.r1 - self.r2 * math.cos(v)) * P - math.sin(v) * OUT
